@@ -1,10 +1,13 @@
-use crate::host::Externals;
-use crate::isa;
-use crate::module::ModuleInstance;
-use crate::runner::{check_function_args, Interpreter, InterpreterState, StackRecycler};
-use crate::types::ValueType;
-use crate::value::RuntimeValue;
-use crate::{Signature, Trap};
+use crate::{
+    host::Externals,
+    isa,
+    module::ModuleInstance,
+    runner::{check_function_args, Interpreter, InterpreterState, StackRecycler},
+    types::ValueType,
+    value::RuntimeValue,
+    Signature,
+    Trap,
+};
 use alloc::{
     borrow::Cow,
     rc::{Rc, Weak},
