@@ -110,7 +110,7 @@ use alloc::{
     string::{String, ToString},
     vec::Vec,
 };
-use core::fmt;
+use core::fmt::{self, Display};
 #[cfg(feature = "std")]
 use std::error;
 
@@ -145,14 +145,14 @@ impl Trap {
 
 impl fmt::Display for Trap {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Trap: {:?}", self.kind)
+        write!(f, "{}", self.kind())
     }
 }
 
 #[cfg(feature = "std")]
 impl error::Error for Trap {
     fn description(&self) -> &str {
-        "runtime trap"
+        self.kind().trap_message()
     }
 }
 
@@ -197,13 +197,19 @@ pub enum TrapKind {
     /// zero as divider.
     DivisionByZero,
 
+    /// An integer arithmetic operation caused an overflow.
+    ///
+    /// This can happen when:
+    ///
+    /// - Trying to do signed division (or get the remainder) -2<sup>N-1</sup> over -1. This is
+    ///   because the result +2<sup>N-1</sup> isn't representable as a N-bit signed integer.
+    IntegerOverflow,
+
     /// Attempt to make a conversion to an int failed.
     ///
     /// This can happen when:
     ///
-    /// - trying to do signed division (or get the remainder) -2<sup>N-1</sup> over -1. This is
-    ///   because the result +2<sup>N-1</sup> isn't representable as a N-bit signed integer.
-    /// - trying to truncate NaNs, infinity, or value for which the result is out of range into an integer.
+    /// - Trying to truncate NaNs, infinity, or value for which the result is out of range into an integer.
     InvalidConversionToInt,
 
     /// Stack overflow.
@@ -238,6 +244,35 @@ impl TrapKind {
     pub fn is_host(&self) -> bool {
         matches!(self, TrapKind::Host(_))
     }
+
+    /// Returns the trap message as specified by the WebAssembly specification.
+    ///
+    /// # Note
+    ///
+    /// This API is primarily useful for the Wasm spec testsuite but might have
+    /// other uses since it avoid heap memory allocation in certain cases.
+    pub fn trap_message(&self) -> &'static str {
+        match self {
+            TrapKind::Unreachable => "unreachable",
+            TrapKind::MemoryAccessOutOfBounds => "out of bounds memory access",
+            TrapKind::TableAccessOutOfBounds => "undefined element",
+            TrapKind::ElemUninitialized => "uninitialized element",
+            TrapKind::DivisionByZero => "integer divide by zero",
+            TrapKind::IntegerOverflow => "integer overflow",
+            TrapKind::InvalidConversionToInt => "invalid conversion to integer",
+            TrapKind::StackOverflow => "call stack exhausted",
+            TrapKind::UnexpectedSignature => "indirect call type mismatch",
+
+            // Note: The below trap message is not further specified by the Wasm spec.
+            TrapKind::Host(_) => "host trap",
+        }
+    }
+}
+
+impl Display for TrapKind {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.trap_message())
+    }
 }
 
 /// Internal interpreter error.
@@ -423,21 +458,22 @@ mod value;
 #[cfg(test)]
 mod tests;
 
-pub use self::func::{FuncInstance, FuncInvocation, FuncRef, ResumableError};
-pub use self::global::{GlobalInstance, GlobalRef};
-pub use self::host::{Externals, HostError, NopExternals, RuntimeArgs};
-pub use self::imports::{ImportResolver, ImportsBuilder, ModuleImportResolver};
-pub use self::memory::{MemoryInstance, MemoryRef, LINEAR_MEMORY_PAGE_SIZE};
-pub use self::module::{ExternVal, ModuleInstance, ModuleRef, NotStartedModuleRef};
-pub use self::runner::{StackRecycler, DEFAULT_CALL_STACK_LIMIT, DEFAULT_VALUE_STACK_LIMIT};
-pub use self::table::{TableInstance, TableRef};
-pub use self::types::{GlobalDescriptor, MemoryDescriptor, Signature, TableDescriptor, ValueType};
-pub use self::value::{Error as ValueError, FromRuntimeValue, LittleEndianConvert, RuntimeValue};
+pub use self::{
+    func::{FuncInstance, FuncInvocation, FuncRef, ResumableError},
+    global::{GlobalInstance, GlobalRef},
+    host::{Externals, HostError, NopExternals, RuntimeArgs},
+    imports::{ImportResolver, ImportsBuilder, ModuleImportResolver},
+    memory::{MemoryInstance, MemoryRef, LINEAR_MEMORY_PAGE_SIZE},
+    module::{ExternVal, ModuleInstance, ModuleRef, NotStartedModuleRef},
+    runner::{StackRecycler, DEFAULT_CALL_STACK_LIMIT, DEFAULT_VALUE_STACK_LIMIT},
+    table::{TableInstance, TableRef},
+    types::{GlobalDescriptor, MemoryDescriptor, Signature, TableDescriptor, ValueType},
+    value::{FromRuntimeValue, LittleEndianConvert, RuntimeValue},
+};
 
 /// WebAssembly-specific sizes and units.
 pub mod memory_units {
-    pub use memory_units::wasm32::*;
-    pub use memory_units::{size_of, ByteSize, Bytes, RoundUpTo};
+    pub use memory_units::{size_of, wasm32::*, ByteSize, Bytes, RoundUpTo};
 }
 
 /// Deserialized module prepared for instantiation.
