@@ -1,22 +1,32 @@
-use crate::func::{FuncBody, FuncInstance, FuncRef};
-use crate::global::{GlobalInstance, GlobalRef};
-use crate::host::Externals;
-use crate::imports::ImportResolver;
-use crate::memory::MemoryRef;
-use crate::memory_units::Pages;
-use crate::runner::StackRecycler;
-use crate::table::TableRef;
-use crate::types::{GlobalDescriptor, MemoryDescriptor, TableDescriptor};
-use crate::{Error, MemoryInstance, Module, RuntimeValue, Signature, TableInstance, Trap};
-use alloc::collections::BTreeMap;
+use crate::{
+    func::{FuncBody, FuncInstance, FuncRef},
+    global::{GlobalInstance, GlobalRef},
+    host::Externals,
+    imports::ImportResolver,
+    memory::MemoryRef,
+    memory_units::Pages,
+    runner::StackRecycler,
+    table::TableRef,
+    types::{GlobalDescriptor, MemoryDescriptor, TableDescriptor},
+    Error,
+    MemoryInstance,
+    Module,
+    RuntimeValue,
+    Signature,
+    TableInstance,
+    Trap,
+};
 use alloc::{
     borrow::ToOwned,
+    collections::BTreeMap,
     rc::Rc,
     string::{String, ToString},
     vec::Vec,
 };
-use core::cell::{Ref, RefCell};
-use core::fmt;
+use core::{
+    cell::{Ref, RefCell},
+    fmt,
+};
 use parity_wasm::elements::{External, InitExpr, Instruction, Internal, ResizableLimits, Type};
 use validation::{DEFAULT_MEMORY_INDEX, DEFAULT_TABLE_INDEX};
 
@@ -831,10 +841,12 @@ pub fn check_limits(limits: &ResizableLimits) -> Result<(), Error> {
 #[cfg(test)]
 mod tests {
     use super::{ExternVal, ModuleInstance};
-    use crate::func::FuncInstance;
-    use crate::imports::ImportsBuilder;
-    use crate::tests::parse_wat;
-    use crate::types::{Signature, ValueType};
+    use crate::{
+        func::FuncInstance,
+        imports::ImportsBuilder,
+        tests::parse_wat,
+        types::{Signature, ValueType},
+    };
 
     #[should_panic]
     #[test]
