@@ -1,15 +1,10 @@
-use crate::nan_preserving_float::{F32, F64};
-use crate::types::ValueType;
-use crate::TrapKind;
+use crate::{
+    nan_preserving_float::{F32, F64},
+    types::ValueType,
+    TrapKind,
+};
 use core::{f32, i32, i64, u32, u64};
 
-/// Error for `LittleEndianConvert`
-#[derive(Debug)]
-pub enum Error {
-    /// The buffer is too short for the type being deserialized
-    InvalidLittleEndianBuffer,
-}
-
 /// Runtime representation of a value.
 ///
 /// Wasm code manipulate values of the four basic value types:
@@ -75,17 +70,59 @@ pub trait TransmuteInto<T> {
     fn transmute_into(self) -> T;
 }
 
-/// Convert from and to little endian.
-pub trait LittleEndianConvert
-where
-    Self: Sized,
-{
-    /// Convert to little endian buffer.
-    fn into_little_endian(self, buffer: &mut [u8]);
-    /// Convert from little endian buffer.
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error>;
+/// Types that can be converted from and to little endian bytes.
+pub trait LittleEndianConvert {
+    /// The little endian bytes representation.
+    type Bytes: Default + AsRef<[u8]> + AsMut<[u8]>;
+
+    /// Converts `self` into little endian bytes.
+    fn into_le_bytes(self) -> Self::Bytes;
+
+    /// Converts little endian bytes into `Self`.
+    fn from_le_bytes(bytes: Self::Bytes) -> Self;
 }
 
+macro_rules! impl_little_endian_convert_primitive {
+    ( $($primitive:ty),* $(,)? ) => {
+        $(
+            impl LittleEndianConvert for $primitive {
+                type Bytes = [::core::primitive::u8; ::core::mem::size_of::<$primitive>()];
+
+                fn into_le_bytes(self) -> Self::Bytes {
+                    <$primitive>::to_le_bytes(self)
+                }
+
+                fn from_le_bytes(bytes: Self::Bytes) -> Self {
+                    <$primitive>::from_le_bytes(bytes)
+                }
+            }
+        )*
+    };
+}
+impl_little_endian_convert_primitive!(u8, u16, u32, u64, i8, i16, i32, i64, f32, f64);
+
+macro_rules! impl_little_endian_convert_float {
+    ( $( struct $float_ty:ident($uint_ty:ty); )* $(,)? ) => {
+        $(
+            impl LittleEndianConvert for $float_ty {
+                type Bytes = <$uint_ty as LittleEndianConvert>::Bytes;
+
+                fn into_le_bytes(self) -> Self::Bytes {
+                    <$uint_ty>::into_le_bytes(self.to_bits())
+                }
+
+                fn from_le_bytes(bytes: Self::Bytes) -> Self {
+                    Self::from_bits(<$uint_ty>::from_le_bytes(bytes))
+                }
+            }
+        )*
+    };
+}
+impl_little_endian_convert_float!(
+    struct F32(u32);
+    struct F64(u64);
+);
+
 /// Arithmetic operations.
 pub trait ArithmeticOps<T> {
     /// Add two values.
@@ -130,6 +167,12 @@ pub trait Float<T>: ArithmeticOps<T> {
     fn nearest(self) -> T;
     /// Takes the square root of a number.
     fn sqrt(self) -> T;
+    /// Returns `true` if the sign of the number is positive.
+    #[allow(clippy::wrong_self_convention)]
+    fn is_sign_positive(self) -> bool;
+    /// Returns `true` if the sign of the number is negative.
+    #[allow(clippy::wrong_self_convention)]
+    fn is_sign_negative(self) -> bool;
     /// Returns the minimum of the two numbers.
     fn min(self, other: T) -> T;
     /// Returns the maximum of the two numbers.
@@ -370,10 +413,13 @@ macro_rules! impl_try_truncate_into {
         impl TryTruncateInto<$into, TrapKind> for $from {
             fn try_truncate_into(self) -> Result<$into, TrapKind> {
                 // Casting from a float to an integer will round the float towards zero
+                if self.is_nan() {
+                    return Err(TrapKind::InvalidConversionToInt);
+                }
                 num_rational::BigRational::from_float(self)
                     .map(|val| val.to_integer())
                     .and_then(|val| $to_primitive(&val))
-                    .ok_or(TrapKind::InvalidConversionToInt)
+                    .ok_or(TrapKind::IntegerOverflow)
             }
         }
     };
@@ -563,171 +609,6 @@ impl TransmuteInto<i64> for u64 {
     }
 }
 
-impl LittleEndianConvert for i8 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer[0] = self as u8;
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        buffer
-            .get(0)
-            .map(|v| *v as i8)
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for u8 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer[0] = self;
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        buffer
-            .get(0)
-            .cloned()
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for i16 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 2];
-        buffer
-            .get(0..2)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_le_bytes(res)
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for u16 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 2];
-        buffer
-            .get(0..2)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_le_bytes(res)
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for i32 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 4];
-        buffer
-            .get(0..4)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_le_bytes(res)
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for u32 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 4];
-        buffer
-            .get(0..4)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_le_bytes(res)
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for i64 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 8];
-        buffer
-            .get(0..8)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_le_bytes(res)
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for f32 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_bits().to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 4];
-        buffer
-            .get(0..4)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_bits(u32::from_le_bytes(res))
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for f64 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        buffer.copy_from_slice(&self.to_bits().to_le_bytes());
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        let mut res = [0u8; 8];
-        buffer
-            .get(0..8)
-            .map(|s| {
-                res.copy_from_slice(s);
-                Self::from_bits(u64::from_le_bytes(res))
-            })
-            .ok_or(Error::InvalidLittleEndianBuffer)
-    }
-}
-
-impl LittleEndianConvert for F32 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        (self.to_bits() as i32).into_little_endian(buffer)
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        i32::from_little_endian(buffer).map(|val| Self::from_bits(val as _))
-    }
-}
-
-impl LittleEndianConvert for F64 {
-    fn into_little_endian(self, buffer: &mut [u8]) {
-        (self.to_bits() as i64).into_little_endian(buffer)
-    }
-
-    fn from_little_endian(buffer: &[u8]) -> Result<Self, Error> {
-        i64::from_little_endian(buffer).map(|val| Self::from_bits(val as _))
-    }
-}
-
 macro_rules! impl_integer_arithmetic_ops {
     ($type: ident) => {
         impl ArithmeticOps<$type> for $type {
@@ -746,7 +627,7 @@ macro_rules! impl_integer_arithmetic_ops {
                 } else {
                     let (result, overflow) = self.overflowing_div(other);
                     if overflow {
-                        Err(TrapKind::InvalidConversionToInt)
+                        Err(TrapKind::IntegerOverflow)
                     } else {
                         Ok(result)
                     }
@@ -827,8 +708,7 @@ mod fmath {
 
 #[cfg(not(feature = "std"))]
 mod fmath {
-    pub use super::libm_adapters::f32;
-    pub use super::libm_adapters::f64;
+    pub use super::libm_adapters::{f32, f64};
 }
 
 // We cannot call the math functions directly, because they are not all available in `core`.
@@ -872,35 +752,44 @@ macro_rules! impl_float {
             fn sqrt(self) -> $type {
                 fmath::$fXX::sqrt($fXX::from(self)).into()
             }
-            // This instruction corresponds to what is sometimes called "minNaN" in other languages.
+            fn is_sign_positive(self) -> bool {
+                $fXX::is_sign_positive($fXX::from(self)).into()
+            }
+            fn is_sign_negative(self) -> bool {
+                $fXX::is_sign_negative($fXX::from(self)).into()
+            }
             fn min(self, other: $type) -> $type {
-                if self.is_nan() {
-                    return self;
+                // The implementation strictly adheres to the mandated behavior for the Wasm specification.
+                // Note: In other contexts this API is also known as: `nan_min`.
+                match (self.is_nan(), other.is_nan()) {
+                    (true, false) => self,
+                    (false, true) => other,
+                    _ => {
+                        // Case: Both values are NaN; OR both values are non-NaN.
+                        if other.is_sign_negative() {
+                            return other.min(self);
+                        }
+                        self.min(other)
+                    }
                 }
-                if other.is_nan() {
-                    return other;
-                }
-
-                self.min(other)
             }
-            // This instruction corresponds to what is sometimes called "maxNaN" in other languages.
             fn max(self, other: $type) -> $type {
-                if self.is_nan() {
-                    return self;
+                // The implementation strictly adheres to the mandated behavior for the Wasm specification.
+                // Note: In other contexts this API is also known as: `nan_max`.
+                match (self.is_nan(), other.is_nan()) {
+                    (true, false) => self,
+                    (false, true) => other,
+                    _ => {
+                        // Case: Both values are NaN; OR both values are non-NaN.
+                        if other.is_sign_positive() {
+                            return other.max(self);
+                        }
+                        self.max(other)
+                    }
                 }
-                if other.is_nan() {
-                    return other;
-                }
-
-                self.max(other)
             }
             fn copysign(self, other: $type) -> $type {
                 use core::mem::size_of;
-
-                if self.is_nan() {
-                    return self;
-                }
-
                 let sign_mask: $iXX = 1 << ((size_of::<$iXX>() << 3) - 1);
                 let self_int: $iXX = self.transmute_into();
                 let other_int: $iXX = other.transmute_into();
@@ -918,11 +807,48 @@ macro_rules! impl_float {
     };
 }
 
+#[test]
+fn wasm_float_min_regression_works() {
+    assert_eq!(
+        Float::min(F32::from(-0.0), F32::from(0.0)).to_bits(),
+        0x8000_0000,
+    );
+    assert_eq!(
+        Float::min(F32::from(0.0), F32::from(-0.0)).to_bits(),
+        0x8000_0000,
+    );
+}
+
+#[test]
+fn wasm_float_max_regression_works() {
+    assert_eq!(
+        Float::max(F32::from(-0.0), F32::from(0.0)).to_bits(),
+        0x0000_0000,
+    );
+    assert_eq!(
+        Float::max(F32::from(0.0), F32::from(-0.0)).to_bits(),
+        0x0000_0000,
+    );
+}
+
 impl_float!(f32, f32, i32);
 impl_float!(f64, f64, i64);
 impl_float!(F32, f32, i32);
 impl_float!(F64, f64, i64);
 
+#[test]
+fn copysign_regression_works() {
+    // This test has been directly extracted from a WebAssembly Specification assertion.
+    use Float as _;
+    assert!(F32::from_bits(0xFFC00000).is_nan());
+    assert_eq!(
+        F32::from_bits(0xFFC00000)
+            .copysign(F32::from_bits(0x0000_0000))
+            .to_bits(),
+        F32::from_bits(0x7FC00000).to_bits()
+    )
+}
+
 #[cfg(not(feature = "std"))]
 mod libm_adapters {
     pub mod f32 {
